Description: 
Success Factors is DFO's HR systems. It needed to be integrated with SVV's archive application called Mime 360 via Geointegration.
The integration itself takes place via Cloud Platform Integration (CPI). The communication towards the archive solution takes place through various API calls.
SuFa - Mime integration is a SOAP based service that exposes the creation of Cases and Journals in SVV Archive and Journal system Mime 360 
It connects to Mime360 through MimeWS. The integration provides reliable integration service to archive and manage HR system activities and documentation 

# Spring profiles

- Profile for this application defines the environment.
- The application consists of 4 profiles which are connected to 4 property files
    - `local` profile = `application-local.yaml`
    - `dev` (STM) profile = `application-stm.yaml`
    - `acceptance test` (ATM) profile = `application-atm.yaml`
    - `prod` profile = `application-prod.yaml`
- `application.yaml` file is default placeholder. Only non-profile related properties should be configured there.
- Build the application using `./mvnw clean install`
- Run the application in specific environment.
  using `java -jar -Dspring.profiles.active=local target/<jarfile>.jar`

## Running locally

//Not using SFTP anymore
- Install Docker (give link to docket site)
- Run `docker-compose up -d sftp` from the root folder to start sftp service
- sftp server will run with credentials on port 2222

- build the application './mvn clean install' OR
-  'mvn clean deploy -DskipTests' to deploy without tests
- run the application 'java -jar -Dspring.profiles.active=local target/integration-0.0.1-SNAPSHOT.jar'

## update jetbrains with correct profile
- Run the application from the Application.Class file. This will fail as you have not set the profile yet.
- Then click on the edit configuration option.
- ![img.png](docs/img.png)
- This adds the local profile in the vm-options
- ![img_1.png](docs/img_1.png)

## Deploying in Atlas
Artrepo plugins are included in the pom.xml file to upload JAR SNAPSHOT in the artifactory 
to deploy using Atlas for STM and ATM env.
**STM (dev):**
- build the application './mvn clean install' OR
- 'mvn clean deploy -DskipTests' to deploy without tests - ![img_1.png](docs/uploadArtfact.JPG)
- login to Atlas CLI in Windows CMD using 'ac login' - ![img_1.png](docs/acLogin.JPG)
- Create a build with the uploaded JAR in the artifactory with the command -
  'ac build geomime -i mimegeo -v 0.0.1-SNAPSHOT -b openjdk8 -U https://artrepo.vegvesen.no/artifactory/libs-snapshot-local/no/vegvesen/ikt/geomime/integration/0.0.1-SNAPSHOT/integration-0.0.1-SNAPSHOT.jar'
  ![img_1.png](docs/createBuild.JPG)
- Run 'ac create deploy geomime -e utvikling -i mimegeo' to deploy ![img_1.png](docs/createDeploy.JPG)
- Check the status and logs in 'https://atlas-api.atlas.vegvesen.no/konsoll/#/iktlosninger/'

**ATM (test):**
- build the application './mvn clean install' OR
- 'mvn clean deploy -DskipTests' to deploy without tests - ![img_1.png](docs/uploadArtfact.JPG)
- login to Atlas CLI in Windows CMD using 'ac login' - ![img_1.png](docs/acLogin.JPG)
- Create a build with the uploaded JAR in the artifactory with the command -
  'ac build geomime -i mimegeo -v 0.0.1-SNAPSHOT -b openjdk8 -U https://artrepo.vegvesen.no/artifactory/libs-snapshot-local/no/vegvesen/ikt/geomime/integration/0.0.1-SNAPSHOT/integration-0.0.1-SNAPSHOT.jar'
  ![img_1.png](docs/createBuild.JPG)
- Run 'ac create deploy geomime -e akseptansetest -i mimegeo' to deploy ![img_1.png](docs/createDeploy.JPG)
- Check the status and logs in 'https://atlas-api.atlas.vegvesen.no/konsoll/#/iktlosninger/'


## Building docker image
// Not used anymore
- A sample Dockerfile is provider.
- Make sure the application works.
- Create and tag docker image `docker build -t svv:001 .` from project root
- Run docker image with a given spring profile `docker run  --env SPRING_PROFILES_ACTIVE=local -p8080:8081 svv:001`

## Running via docker-compose
- this is the preferred way if you want to test the end to end connectivity
- build and tag the docker image as shown in the steps above.
- replace the sftp_host and sftp_port properties in docker-compose.yaml
- run `docker-compose up`
