package no.integration.svv.exception;

import lombok.Data;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

@Data
public class SplunkLogging {

    private static final String SVVGUID = "svvguid";
    private static final String MSG = "msg";
    private static final String USER_ID = "user_id";
    private static final String USER = "user";

    private Map<String, String> logMap;
    private String skilletegn = ";";

    private SplunkLogging() {
        logMap = new LinkedHashMap<String, String>();
    }

    private SplunkLogging(String skilletegn) {
        logMap = new LinkedHashMap<String, String>();
        this.skilletegn = skilletegn;
    }

    /**
     * Spawner en log-streng med ; som skilletegn mellom elementene.
     * <br><br>
     * Eksempel på bruk:
     * logger.info(SplunkLog.log().logid(42).msg("Innkommende request").svvguid(svvguid).lagLogString());
     */
    public static SplunkLogging log()
    {
        return new SplunkLogging();
    }

    public static SplunkLogging log(String skilletegn)
    {
        return new SplunkLogging(skilletegn);
    }

    public String lagLogString()
    {
        String logString = "";

        Set<Map.Entry<String, String>> entrySet = logMap.entrySet();
        for (Map.Entry<String, String> entry : entrySet)
        {
            String tmpLogPart = entry.getKey();
            if (entry.getValue() != null && entry.getValue().trim().contains(" "))
            {
                tmpLogPart += "=\"" + entry.getValue() + "\"";
            } else
            {
                tmpLogPart += "=" + entry.getValue();
            }
            tmpLogPart += skilletegn;
            logString += tmpLogPart;

        }

        return logString;
    }
    public SplunkLogging svvguid(String value) {
        this.logMap.put(SVVGUID, value);
        return this;
    }

    public SplunkLogging msg(String value) {
        this.logMap.put(MSG, value);
        return this;
    }

    public SplunkLogging userId(String value) {
        this.logMap.put(USER_ID, value);
        return this;
    }

    public SplunkLogging user(String value) {
        this.logMap.put(USER, value);
        return this;
    }
    public SplunkLogging med(String key, String value) {
        logMap.put(key, value);
        return this;
    }

}
