package no.integration.svv.service;

import no.integration.svv.properties.WSClientConfigProperties;
import no.vegvesen.saksarkiv.felles.typer._1.JournalpostKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.KvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.NyttVedleggKvitteringType;
import no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.EndreStatusRequestType;
import no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.LeggTilNyttDokumentJournalpostRequestType;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.ObjectFactory;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.OpprettJournalpostRequestType;
import no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.OpprettSaksmappeRequestType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.springframework.ws.soap.client.core.SoapActionCallback;

import javax.xml.bind.JAXBElement;
/**
 @author vijayjadhav
 **/

/**
* Endpoint to MimeWS element mapping to receive response for all operations
* */

@Service
public class SoapClient {

    private static final Logger log = LoggerFactory.getLogger(SoapClient.class);

    private final WebServiceTemplate webServiceTemplate;
    private final WSClientConfigProperties wsClientConfigProperties;

    public SoapClient(final WebServiceTemplate webServiceTemplate,
                      final WSClientConfigProperties wsClientConfigProperties) {
        this.webServiceTemplate = webServiceTemplate;
        this.wsClientConfigProperties = wsClientConfigProperties;
    }

    /**
     * NyJournalPost operation : Marshalling and unmarshalling of objects from the
     integration endpoint to MimeWS to receive response
     * */
    public JournalpostKvitteringType opprettJournalPost(OpprettJournalpostRequestType input) {
        log.info("SoapClient::OpprettJournalpost - Start");

        try {
            ObjectFactory factory = new ObjectFactory();
            JAXBElement<OpprettJournalpostRequestType> rr = factory.createOpprettJournalpostRequest(input);
            Object o = this.webServiceTemplate.marshalSendAndReceive(this.
                    wsClientConfigProperties.getLocationURI() + "/journalpost", rr, new
                    SoapActionCallback("urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/opprett"));

            JAXBElement<JournalpostKvitteringType> jpkt = (JAXBElement<JournalpostKvitteringType>) o;
            log.info("SoapClient::OpprettJournalpost - End");
            return jpkt.getValue();
        } catch (Exception e) {
            log.error("SoapClient::OpprettJournalpost - Exception ", e);
//            log.error(String.valueOf(SplunkLogging.log().med("MimeWS Response","SoapClient::OpprettJournalpost")));
            throw e;
        }
    }

    /**
     * NySaksmappe operation : Marshalling and unmarshalling of objects from the
     integration endpoint to MimeWS to receive response
     * */
    public MappeKvitteringType opprettSaksmappe(OpprettSaksmappeRequestType input) {
        log.info("SoapClient::opprettSaksmappe - Start");
        try {
            no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.ObjectFactory fac = new no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.ObjectFactory();
            JAXBElement<OpprettSaksmappeRequestType> rr = fac.createOpprettSaksmappeRequest(input);
            Object o = this.webServiceTemplate.marshalSendAndReceive(this.wsClientConfigProperties.getLocationURI() + "/saksmappe", rr,
                    new SoapActionCallback("urn:no:vegvesen:saksarkiv:saksmappe:tjeneste:saksmappe-1/SaksmappeSoapPortType/opprett"));
            JAXBElement<MappeKvitteringType> mkt = (JAXBElement<MappeKvitteringType>) o;
            log.info("SoapClient::opprettSaksmappe - End");
            return mkt.getValue();
        } catch (Exception e) {
            log.error("SoapClient::opprettSaksmappe - Exception ", e);
//            log.error(String.valueOf(SplunkLogging.log().med("MimeWS Response","SoapClient::opprettSaksmappe")));
            throw e;
        }
    }

    /**
     * NyDokument operation : Marshalling and unmarshalling of objects from the
     integration endpoint to MimeWS to receive response
     * */
    public NyttVedleggKvitteringType opprettDokument(LeggTilNyttDokumentJournalpostRequestType input) {
        log.info("SoapClient::opprettDokument - Start");
        try {
            no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.ObjectFactory factory = new no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.ObjectFactory();
            JAXBElement<LeggTilNyttDokumentJournalpostRequestType> rr = factory.createLeggTilNyttDokumentJournalpostRequest(input);
            Object o = this.webServiceTemplate.marshalSendAndReceive(this.wsClientConfigProperties.getLocationURI() + "/journalpost", rr,
                    new SoapActionCallback("urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/leggTilNyttDokument"));
            JAXBElement<NyttVedleggKvitteringType> mkt = (JAXBElement<NyttVedleggKvitteringType>) o;
            log.info("SoapClient::opprettDokument - End");
            return mkt.getValue();
        } catch (Exception e) {
            log.error("SoapClient::opprettDokument - Exception ", e);
//            log.error(String.valueOf(SplunkLogging.log().med("MimeWS Response","SoapClient::opprettDokument")));
            throw e;
        }
    }

    /**
     * NyjournalPostStatus operation : Marshalling and unmarshalling of objects from the
     integration endpoint to MimeWS to receive response
     * */
    public KvitteringType opprettJPStatus(EndreStatusRequestType input) {
        log.info("SoapClient::opprettJPStatus - Start");
        try {
            no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.ObjectFactory factory = new no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.ObjectFactory();
            JAXBElement<EndreStatusRequestType> rr = factory.createEndreStatusRequest(input);
            Object o = this.webServiceTemplate.marshalSendAndReceive(this.wsClientConfigProperties.getLocationURI() + "/journalpost", rr,
                    new SoapActionCallback("urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/endreStatus"));
            JAXBElement<KvitteringType> mkt = (JAXBElement<KvitteringType>) o;
            log.info("SoapClient::opprettJPStatus - End");
            return mkt.getValue();
        } catch (Exception e) {
            log.error("SoapClient::opprettJPStatus - Exception ", e);
//            log.error(String.valueOf(SplunkLogging.log().med("MimeWS Response","SoapClient::opprettJPStatus")));
            throw e;
        }
    }

}
