package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.felles.xml_schema._2012_01.Saksnummer;
import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Journalnummer;
import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Journalpost;
import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Korrespondansepart;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpostResponse;
import no.geointegrasjon.rep.felles.adresse.xml_schema._2012_01.EnkelAdresseListe;
import no.geointegrasjon.rep.felles.adresse.xml_schema._2012_01.Epost;
import no.geointegrasjon.rep.felles.adresse.xml_schema._2012_01.Faks;
import no.geointegrasjon.rep.felles.adresse.xml_schema._2012_01.Telefon;
import no.integration.svv.exception.DataMappingException;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.felles.cct._1.Adresse;
import no.vegvesen.saksarkiv.felles.typer._1.*;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.OpprettJournalpostRequestType;

import java.math.BigInteger;
import java.util.Objects;

import static no.vegvesen.saksarkiv.felles.enkle._1.HandlingType.LEGG_TIL;
import static no.vegvesen.saksarkiv.felles.enkle._1.JournalposttypeType.*;
import static no.vegvesen.saksarkiv.felles.enkle._1.KorrespondanseparttypeType.INTERN;
import static no.vegvesen.saksarkiv.felles.enkle._1.KorrespondanseparttypeType.PERSON;
/**
 * @author vijayjadhav
 **/

/**
 * Geo-Endpoint to MimeWS Request mapping for NyJournalPost operation
 */
public class JournalPost {

    /**
     * Request mapping from the endpoint to MimeWS server
     */
    public static OpprettJournalpostRequestType endpointToMimeOppJPT(final NyJournalpost input) throws ValidationException {

        RequestValidator.validateJournalpostRequest(input);

        OpprettJournalpostRequestType res = new OpprettJournalpostRequestType();
        OpprettSammenstiltJournalpostType osjpt = new OpprettSammenstiltJournalpostType();

        JournalpostBasisMetadataType jpbmd = new JournalpostBasisMetadataType();

        if (input.getJournalpost() != null) {
            jpbmd.setTittel(input.getJournalpost().getTittel());
            jpbmd.setOffentligTittel(input.getJournalpost().getOffentligTittel());
            FagsystemType fagsystemType = getFagSystemMapping(input);
            res.setFagsystem(fagsystemType);

            if (!input.getJournalpost().isSkjermetTittel()) {
                jpbmd.setOffentligTittel("");
            } else jpbmd.setOffentligTittel(input.getJournalpost().getOffentligTittel());

            /*
            * If both ID and saksnr is input, this has higher priority and saksnr is to be dropped and this value to be used
            *
            */

            Saksnummer saksnummer = input.getJournalpost().getSaksnr();
            MappeIdentifikatorType mappeIdentifikatorType = new MappeIdentifikatorType();
            MappeIdType mappeIdType = new MappeIdType();

            if (input.getJournalpost().getReferanseSakSystemID() != null && input.getJournalpost().getReferanseSakSystemID().getSystemID() != null) {
                String id = input.getJournalpost().getReferanseSakSystemID().getSystemID().getId();
                if (id != null) {
                    mappeIdType.setSaksnummer((id.substring(0, 2)) + "/" + (id.substring(2, id.length())));
                }
            }

                if (input.getJournalpost().getReferanseSakSystemID().getSystemID().getId() == null ||
                        Objects.equals(input.getJournalpost().getReferanseSakSystemID().getSystemID().getId(), "")) {
                    if (saksnummer != null) {
                    if (saksnummer.getSakssekvensnummer() != null && saksnummer.getSaksaar() != null) {
                        mappeIdType.setSaksnummer((saksnummer.getSaksaar().toString() + "/" + saksnummer.getSakssekvensnummer().toString()));
                    }
                }
            }


            mappeIdentifikatorType.setMappeId(mappeIdType);
            jpbmd.setMappe(mappeIdentifikatorType);


            TilgangType tilgangType = new TilgangType();
            TillatelseType tillatelseType = new TillatelseType();
            TilgangsGruppeTilgangType tilgangsGruppeTilgangType = new TilgangsGruppeTilgangType();
            if (input.getJournalpost().getTilleggsinformasjon().getListe() != null &&
                    input.getJournalpost().getTilleggsinformasjon().getListe().size() > 0) {
                input.getJournalpost().getTilleggsinformasjon().getListe().forEach(item -> {
                    tilgangsGruppeTilgangType.setGruppenavn(item.getTilgangsgruppeNavn());
                });
                tillatelseType.setTilgangsgruppe(tilgangsGruppeTilgangType);
                tilgangType.setTillatelse(tillatelseType);
                osjpt.setTilgang(tilgangType);
            }

            JournalpostMetadataType journalpostMetadataType = new JournalpostMetadataType();
            if (Objects.equals(input.getJournalpost().getJournalposttype().getKodeverdi(), "I")) {
                journalpostMetadataType.setJournalpostType(INNGAAENDE);
            } else if (Objects.equals(input.getJournalpost().getJournalposttype().getKodeverdi(), "U")) {
                journalpostMetadataType.setJournalpostType(UTGAAENDE);
            } else if (Objects.equals(input.getJournalpost().getJournalposttype().getKodeverdi(), "N")) {
                journalpostMetadataType.setJournalpostType(NOTAT_MED_OPPFOLGING);
            } else if (Objects.equals(input.getJournalpost().getJournalposttype().getKodeverdi(), "X")) {
                journalpostMetadataType.setJournalpostType(NOTAT_UTEN_OPPFOLGING);
            }

            if (input.getJournalpost().getKorrespondansepart() != null &&
                    input.getJournalpost().getKorrespondansepart().getListe().size() > 0) {
                SkjermingRequestType skjermingRequestType = new SkjermingRequestType();
                if (input.getJournalpost().getSkjerming() != null) {
                    HjemmelRequestType hjemmelRequestType = new HjemmelRequestType();
                    SkjermingskodeRequestType skjermingskodeRequestType = new SkjermingskodeRequestType();

                    if (input.getJournalpost().getSkjerming().getTilgangsrestriksjon() != null) {


                        //ToDo: Need functional inputs
//                        if (Objects.equals(input.getJournalpost().getSkjerming().getTilgangsrestriksjon().getKodeverdi(), "U")){
//                            skjermingskodeRequestType.setNavn("Uskjermet");
//                        }
//                        if (Objects.equals(input.getJournalpost().getSkjerming().getTilgangsrestriksjon().getKodeverdi(), "UO")){
//                            skjermingskodeRequestType.setNavn("Unntatt offentlighet");
//                        }

                        skjermingskodeRequestType.setNavn(input.getJournalpost().getSkjerming().getTilgangsrestriksjon().getKodeverdi());
                        {
                            /*
                            * Either all list items is shielded or no one is:
                                For ex. (i=init,  liste.skjermetKorrespondansepart(i) == True, i++) {All list-elements should be set to True}
                            * */
                            Boolean isErGyldigTrue = true;
                            for (Korrespondansepart korrItem : input.getJournalpost().getKorrespondansepart().getListe()) {
                                if (korrItem.isSkjermetKorrespondansepart()) {
                                    skjermingskodeRequestType.setKorrespondanseparterSkjermet(isErGyldigTrue);
                                }
                            }
                        }
                    }
                    hjemmelRequestType.setNavn(input.getJournalpost().getSkjerming().getSkjermingshjemmel());
                    skjermingskodeRequestType.setHjemmel(hjemmelRequestType);
                    skjermingRequestType.setKode(skjermingskodeRequestType);
                    skjermingRequestType.setHandling(LEGG_TIL);
                }
                jpbmd.setSkjerming(skjermingRequestType);

                KorrespondansepartMetadataType korrespondansepartMetadataType = new KorrespondansepartMetadataType();
                KorrespondansepartEnhetType korrespondansepartEnhetType = new KorrespondansepartEnhetType();
                AdressatIdentifikatorType adressatIdentifikatorType = new AdressatIdentifikatorType();
                AnsvarType ansvarType = new AnsvarType();
                BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
                AvsenderType avsenderType = new AvsenderType();
                MottakerType mottakerType = new MottakerType();
                AnsvarsstedType ansvarsstedType = new AnsvarsstedType();
                String jpType = input.getJournalpost().getJournalposttype().getKodeverdi();

/**
for type M in U, N or X journalpost types, the fields administrativEnhetInit
 should map to Mottaker.Identifikator.Ansvarskode and saksbehandlerInit should map to Mottaker.Identifikator.Brukerident
* */

                if (jpType.equalsIgnoreCase("U")) {
                    boolean isASet = false;
                    boolean isMSet = false;
                    for (Korrespondansepart item : input.getJournalpost().getKorrespondansepart().getListe()) {


                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("A") && !isASet) {
                            //logic to set ansvar
                            ansvarsstedType.setAnsvarskode(item.getAdministrativEnhetInit());
                            brukeridentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            ansvarType.setAnsvarligEnhet(ansvarsstedType);
                            ansvarType.setAnsvarligPerson(brukeridentifikatorType);
                            isASet = true;
                            jpbmd.setAnsvar(ansvarType);
                        } else if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("M") && !isMSet) {
                            if (item.getForsendelsesmaate() != null) {
                                korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                            }
                            if (item.getKontakt() != null) {

                                if (item.getKontakt().getElektroniskeAdresser() != null &&
                                        item.getKontakt().getElektroniskeAdresser().getListe().size() > 0) {
                                    // Casted to map
                                    item.getKontakt().getElektroniskeAdresser().getListe().forEach(elektroniskAdresse -> {
                                        if (elektroniskAdresse instanceof Epost) {
                                            korrespondansepartMetadataType.setEpost(((Epost) elektroniskAdresse).getEpostadresse());
                                        }
                                        if (elektroniskAdresse instanceof Faks) {
                                            korrespondansepartMetadataType.setFaks(((Faks) elektroniskAdresse).getFaksnummer());
                                        }
                                        if (elektroniskAdresse instanceof Telefon) {
                                            korrespondansepartMetadataType.setTelefon(((Telefon) elektroniskAdresse).getTelefonnummer());
                                        }
                                    });
                                }
                                Adresse adresseOutput = new Adresse();
                                EnkelAdresseListe addressGetInput = item.getKontakt().getAdresser();
                                if (addressGetInput != null) {

                                    if (addressGetInput.getListe() != null && addressGetInput.getListe().size() > 0) {

                                        addressGetInput.getListe().forEach(item2 -> {
                                            if (item2.getPostadresse() != null) {
                                                adresseOutput.setPostnummer(item2.getPostadresse().getPostnummer());
                                                adresseOutput.setPoststed(item2.getPostadresse().getPoststed());
                                            }
                                            if (item2.getAdresselinje1() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje1());
                                            }
                                            if (item2.getAdresselinje2() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje2());
                                            }
                                            if (item2.getLandkode() != null && addressGetInput.getListe().size() > 0) {
                                                adresseOutput.setLand(item2.getLandkode().getKodeverdi());
                                            }
                                        });

                                        if (item.getForsendelsesmaate() != null) {
                                            korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                                        }
                                    }
                                }
                                korrespondansepartMetadataType.setPostadresse(adresseOutput);
                                korrespondansepartMetadataType.setKorrespondanseparttype(INTERN);
                                korrespondansepartMetadataType.setNavn(item.getKontakt().getNavn());
                            }

                            //logic to set Korrespondansepart
                            korrespondansepartMetadataType.setKorrespondansepartid(item.getSystemID());
                            adressatIdentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            adressatIdentifikatorType.setAnsvarskode(item.getAdministrativEnhetInit());
                            mottakerType.setIdentifikator(adressatIdentifikatorType);
                            mottakerType.setMetadata(korrespondansepartMetadataType);
                            journalpostMetadataType.getMottaker().add(mottakerType);
                            isMSet = true;
                            osjpt.setJournalpostMetadata(journalpostMetadataType);
                        }

                        if (isASet && isMSet) {
                            break;
                        }
                    }
                    if (jpType.equalsIgnoreCase("U") && !isASet) {
                        throw new DataMappingException("journalposttype", jpType, "Avsender is not set for journalPostType - " + jpType);
                    }
                    if (jpType.equalsIgnoreCase("U") && !isMSet) {
                        throw new DataMappingException("journalposttype", jpType, "Mottaker is not set for journalPostType - " + jpType);
                    }
                }
                if (jpType.equalsIgnoreCase("N")) {
                    boolean isASet = false;
                    boolean isMSet = false;
                    for (Korrespondansepart item : input.getJournalpost().getKorrespondansepart().getListe()) {
                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("A") && !isASet) {
                            //logic to set ansvar
                            ansvarsstedType.setAnsvarskode(item.getAdministrativEnhetInit());
                            brukeridentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            ansvarType.setAnsvarligEnhet(ansvarsstedType);
                            ansvarType.setAnsvarligPerson(brukeridentifikatorType);
                            isASet = true;
                            jpbmd.setAnsvar(ansvarType);
                        }
                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("M") && !isMSet) {
                            if (item.getForsendelsesmaate() != null) {
                                korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                            }
                            if (item.getKontakt() != null) {

                                if (item.getKontakt().getElektroniskeAdresser() != null &&
                                        item.getKontakt().getElektroniskeAdresser().getListe().size() > 0) {
                                    // Casted to map
                                    item.getKontakt().getElektroniskeAdresser().getListe().forEach(elektroniskAdresse -> {
                                        if (elektroniskAdresse instanceof Epost) {
                                            korrespondansepartMetadataType.setEpost(((Epost) elektroniskAdresse).getEpostadresse());
                                        }
                                        if (elektroniskAdresse instanceof Faks) {
                                            korrespondansepartMetadataType.setFaks(((Faks) elektroniskAdresse).getFaksnummer());
                                        }
                                        if (elektroniskAdresse instanceof Telefon) {
                                            korrespondansepartMetadataType.setTelefon(((Telefon) elektroniskAdresse).getTelefonnummer());
                                        }
                                    });
                                }
                                Adresse adresseOutput = new Adresse();
                                EnkelAdresseListe addressGetInput = item.getKontakt().getAdresser();
                                if (addressGetInput != null) {

                                    if (addressGetInput.getListe() != null && addressGetInput.getListe().size() > 0) {

                                        addressGetInput.getListe().forEach(item2 -> {
                                            if (item2.getPostadresse() != null) {
                                                adresseOutput.setPostnummer(item2.getPostadresse().getPostnummer());
                                                adresseOutput.setPoststed(item2.getPostadresse().getPoststed());
                                            }
                                            if (item2.getAdresselinje1() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje1());
                                            }
                                            if (item2.getAdresselinje2() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje2());
                                            }
                                            if (item2.getLandkode() != null && addressGetInput.getListe().size() > 0) {
                                                adresseOutput.setLand(item2.getLandkode().getKodeverdi());
                                            }
                                        });

                                        if (item.getForsendelsesmaate() != null) {
                                            korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                                        }
                                    }
                                }
                                korrespondansepartMetadataType.setPostadresse(adresseOutput);
                                korrespondansepartMetadataType.setKorrespondanseparttype(INTERN);
                                korrespondansepartMetadataType.setNavn(item.getKontakt().getNavn());
                            }
                            //logic to set Korrespondansepart

                            korrespondansepartMetadataType.setKorrespondansepartid(item.getSystemID());
                            adressatIdentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            adressatIdentifikatorType.setAnsvarskode(item.getAdministrativEnhetInit());
                            mottakerType.setIdentifikator(adressatIdentifikatorType);
                            mottakerType.setMetadata(korrespondansepartMetadataType);
                            journalpostMetadataType.getMottaker().add(mottakerType);
                            isMSet = true;
                        }
                        osjpt.setJournalpostMetadata(journalpostMetadataType);
                        if (isASet && isMSet) {
                            break;
                        }
                    }
                    if (jpType.equalsIgnoreCase("N") && !isASet) {
                        throw new DataMappingException("journalposttype", jpType, "Avsender is not set for journalPostType - " + jpType);
                    }
                    if (jpType.equalsIgnoreCase("N") && !isMSet) {
                        throw new DataMappingException("journalposttype", jpType, "Mottaker is not set for journalPostType - " + jpType);
                    }
                }
                if (jpType.equalsIgnoreCase("I")) {

                    boolean isMSet = false;
                    boolean isASet = false;
                    avsenderType.setErMedAvsender(true);
                    for (Korrespondansepart item : input.getJournalpost().getKorrespondansepart().getListe()) {
                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("M") && !isMSet) {
                            // logic to set ansvar
                            ansvarsstedType.setAnsvarskode(item.getAdministrativEnhetInit());
                            brukeridentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            ansvarType.setAnsvarligEnhet(ansvarsstedType);
                            ansvarType.setAnsvarligPerson(brukeridentifikatorType);
                            isMSet = true;
                            jpbmd.setAnsvar(ansvarType);
                        }

                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("A") && !isASet) {

                            if (item.getForsendelsesmaate() != null) {
                                korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                            }
                            if (item.getKontakt() != null) {

                                if (item.getKontakt().getElektroniskeAdresser() != null &&
                                        item.getKontakt().getElektroniskeAdresser().getListe().size() > 0) {
                                    // Casted to map
                                    item.getKontakt().getElektroniskeAdresser().getListe().forEach(elektroniskAdresse -> {
                                        if (elektroniskAdresse instanceof Epost) {
                                            korrespondansepartMetadataType.setEpost(((Epost) elektroniskAdresse).getEpostadresse());
                                        }
                                        if (elektroniskAdresse instanceof Faks) {
                                            korrespondansepartMetadataType.setFaks(((Faks) elektroniskAdresse).getFaksnummer());
                                        }
                                        if (elektroniskAdresse instanceof Telefon) {
                                            korrespondansepartMetadataType.setTelefon(((Telefon) elektroniskAdresse).getTelefonnummer());
                                        }
                                    });
                                }
                                Adresse adresseOutput = new Adresse();
                                EnkelAdresseListe addressGetInput = item.getKontakt().getAdresser();
                                if (addressGetInput != null) {

                                    if (addressGetInput.getListe() != null && addressGetInput.getListe().size() > 0) {

                                        addressGetInput.getListe().forEach(item2 -> {
                                            if (item2.getPostadresse() != null) {
                                                adresseOutput.setPostnummer(item2.getPostadresse().getPostnummer());
                                                adresseOutput.setPoststed(item2.getPostadresse().getPoststed());
                                            }
                                            if (item2.getAdresselinje1() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje1());
                                            }
                                            if (item2.getAdresselinje2() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje2());
                                            }
                                            if (item2.getLandkode() != null && addressGetInput.getListe().size() > 0) {
                                                adresseOutput.setLand(item2.getLandkode().getKodeverdi());
                                            }
                                        });

                                        if (item.getForsendelsesmaate() != null) {
                                            korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                                        }
                                    }
                                }
                                korrespondansepartMetadataType.setPostadresse(adresseOutput);

                                korrespondansepartMetadataType.setKorrespondanseparttype(PERSON);
                                korrespondansepartMetadataType.setNavn(item.getKontakt().getNavn());

                            }
                            // logic to set Korrespondansepart
                            korrespondansepartMetadataType.setKorrespondansepartid(item.getSystemID());
                            avsenderType.setMetadata(korrespondansepartMetadataType);
                            isASet = true;
                            avsenderType.setErMedAvsender(false);
                        }
                        if (isASet && isMSet) {break; }
                    }

                    journalpostMetadataType.getAvsender().add(avsenderType);
                    osjpt.setJournalpostMetadata(journalpostMetadataType);


                    if (jpType.equalsIgnoreCase("I") && !isMSet) {
                        throw new DataMappingException("journalposttype", jpType, "Avsender is not set");
                    }
                    if (jpType.equalsIgnoreCase("I") && !isASet) {
                        throw new DataMappingException("journalposttype", jpType, "Mottaker is not set");
                    }
                }
                if (jpType.equalsIgnoreCase("X")) {
                    boolean isMSet = false;
                    boolean isASet = false;
                    for (Korrespondansepart item : input.getJournalpost().getKorrespondansepart().getListe()) {
                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("A") && !isASet) {
                            // logic to set ansvar

                            ansvarsstedType.setAnsvarskode(item.getAdministrativEnhetInit());
                            brukeridentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            ansvarType.setAnsvarligEnhet(ansvarsstedType);
                            ansvarType.setAnsvarligPerson(brukeridentifikatorType);
                            isASet = true;
                            jpbmd.setAnsvar(ansvarType);
                        }
                        if (item.getKorrespondanseparttype().getKodeverdi().equalsIgnoreCase("M") && !isMSet) {

                            if (item.getForsendelsesmaate() != null) {
                                korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                            }
                            if (item.getKontakt() != null) {

                                if (item.getKontakt().getElektroniskeAdresser() != null &&
                                        item.getKontakt().getElektroniskeAdresser().getListe().size() > 0) {
                                    // Casted to map
                                    item.getKontakt().getElektroniskeAdresser().getListe().forEach(elektroniskAdresse -> {
                                        if (elektroniskAdresse instanceof Epost) {
                                            korrespondansepartMetadataType.setEpost(((Epost) elektroniskAdresse).getEpostadresse());
                                        }
                                        if (elektroniskAdresse instanceof Faks) {
                                            korrespondansepartMetadataType.setFaks(((Faks) elektroniskAdresse).getFaksnummer());
                                        }
                                        if (elektroniskAdresse instanceof Telefon) {
                                            korrespondansepartMetadataType.setTelefon(((Telefon) elektroniskAdresse).getTelefonnummer());
                                        }
                                    });
                                }
                                Adresse adresseOutput = new Adresse();
                                EnkelAdresseListe addressGetInput = item.getKontakt().getAdresser();
                                if (addressGetInput != null) {

                                    if (addressGetInput.getListe() != null && addressGetInput.getListe().size() > 0) {

                                        addressGetInput.getListe().forEach(item2 -> {
                                            if (item2.getPostadresse() != null) {
                                                adresseOutput.setPostnummer(item2.getPostadresse().getPostnummer());
                                                adresseOutput.setPoststed(item2.getPostadresse().getPoststed());
                                            }
                                            if (item2.getAdresselinje1() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje1());
                                            }
                                            if (item2.getAdresselinje2() != null) {
                                                adresseOutput.getAdresselinje().add(item2.getAdresselinje2());
                                            }
                                            if (item2.getLandkode() != null && addressGetInput.getListe().size() > 0) {
                                                adresseOutput.setLand(item2.getLandkode().getKodeverdi());
                                            }
                                        });

                                        if (item.getForsendelsesmaate() != null) {
                                            korrespondansepartMetadataType.setEkspederingskanal(item.getForsendelsesmaate().getKodeverdi());
                                        }
                                    }
                                }
                                korrespondansepartMetadataType.setPostadresse(adresseOutput);

                                korrespondansepartMetadataType.setKorrespondanseparttype(INTERN);
                                korrespondansepartMetadataType.setNavn(item.getKontakt().getNavn());

                            }
                            // logic to set Korrespondansepart
                            korrespondansepartMetadataType.setKorrespondansepartid(item.getSystemID());

                            adressatIdentifikatorType.setBrukerident(item.getSaksbehandlerInit());
                            adressatIdentifikatorType.setAnsvarskode(item.getAdministrativEnhetInit());
                            mottakerType.setIdentifikator(adressatIdentifikatorType);
                            mottakerType.setMetadata(korrespondansepartMetadataType);
                            journalpostMetadataType.getMottaker().add(mottakerType);
                            isMSet = true;
                        }
                        osjpt.setJournalpostMetadata(journalpostMetadataType);
                        if (isASet && isMSet) {
                            break;
                        }
                    }

                    if (jpType.equalsIgnoreCase("X") && !isASet) {
                        throw new DataMappingException("journalposttype", jpType, "Avsender is not set");
                    }
                }
            }

            journalpostMetadataType.setDokumentdato(input.getJournalpost().getDokumentetsDato());
            journalpostMetadataType.setForfalldato(input.getJournalpost().getForfallsdato());

            if (input.getJournalpost().getTilleggsinformasjon() != null &&
                    input.getJournalpost().getTilleggsinformasjon().getListe().size() > 0) {
                input.getJournalpost().getTilleggsinformasjon().getListe().forEach(item -> {
                    RegistreringAttributterType registreringAttributterType = new RegistreringAttributterType();
                    MetadataType metadataType = new MetadataType();

                    if (item.getInformasjonstype() != null) {
                        metadataType.setVerdi(item.getInformasjonstype().getKodeverdi());
                        metadataType.setNavn(item.getTilgangsgruppeNavn());
                    }
                    registreringAttributterType.getAttributt().add(metadataType);

                    jpbmd.setRegistreringstypespesifikkeAtributter(registreringAttributterType);
                });
            }
            res.setFagsystem(fagsystemType);
        }

        osjpt.setBasisMetadata(jpbmd);
        osjpt.getVedlegg();
        res.setJournalpost(osjpt);
        return res;

    }

    public static FagsystemType getFagSystemMapping(NyJournalpost input) {
        FagsystemType fagsystemType = new FagsystemType();
        BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
        if (input.getKontekst() != null) {
            fagsystemType.setFagsystemNavn(input.getKontekst().getKlientnavn());
            brukeridentifikatorType.setBrukerident(input.getKontekst().getSluttbruker());
        }
        fagsystemType.setFagsystemSluttbruker(brukeridentifikatorType);
        return fagsystemType;

    }

    /**
     * Response mapping (MimeWS to endpoint)
     */
    public static NyJournalpostResponse nyJournalpostResponseToSuFa(JournalpostKvitteringType mimeResponse) {


        NyJournalpostResponse nyJournalpostResponse = new NyJournalpostResponse();
        Journalpost journalpost = new Journalpost();
        Journalnummer journalnummer = new Journalnummer();

        if (mimeResponse.getRegistrering() != null) {
            journalpost.setSystemID(mimeResponse.getRegistrering().getDokumentId());
            if (mimeResponse.getRegistrering().getJournalpostId() != null) {
                journalpost.setJournalpostnummer(mimeResponse.getRegistrering().getJournalpostId().getLopenummer());
                String saksNummer = mimeResponse.getRegistrering().getJournalpostId().getMappeId().getSaksnummer();
                if (saksNummer != null) {
                    journalnummer.setJournalaar(new BigInteger(saksNummer.substring(0, 2)));
                    journalnummer.setJournalsekvensnummer(new BigInteger(saksNummer.substring(3, saksNummer.length())));
                }
                journalpost.setJournalnummer(journalnummer);

            }
            nyJournalpostResponse.setReturn(journalpost);
        }
        return nyJournalpostResponse;
    }

}

