package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.JournpostSystemID;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.OppdaterJournalpostStatus;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.OppdaterJournalpostStatusResponse;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.BrukeridentifikatorType;
import no.vegvesen.saksarkiv.felles.typer._1.FagsystemType;
import no.vegvesen.saksarkiv.felles.typer._1.KvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.RegistreringIdentifikatorRequestType;
import no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.EndreStatusRequestType;

import static no.vegvesen.saksarkiv.felles.enkle._1.StatusEndringType.JOURNALFOR;
/**
 @author vijayjadhav
 **/
/**
 * Geo-Endpoint to MimeWS Request mapping for NyOppJournalpostStatus operation
 */
public class OppJournalpostStatus {

    /**
     * Request mapping from the endpoint to MimeWS server
     * */

    public static EndreStatusRequestType endpointToMimeOppJPStatus(final OppdaterJournalpostStatus input) {

        RequestValidator.validateJPStatusRequest(input);
        EndreStatusRequestType endreStatusRequestType = new EndreStatusRequestType();

        FagsystemType fagsystemType = getFagSystemMapping(input);
        endreStatusRequestType.setFagsystem(fagsystemType);

        endreStatusRequestType.setJournalpostStatusOperasjon(JOURNALFOR);

        if (input.getJournalpostnokkel() != null) {
            RegistreringIdentifikatorRequestType registreringIdentifikatorRequestType = new RegistreringIdentifikatorRequestType();
            JournpostSystemID journpostSystemID = (JournpostSystemID) input.getJournalpostnokkel();
            registreringIdentifikatorRequestType.setDokumentId((journpostSystemID.getSystemID().getId()));
            endreStatusRequestType.setJournalpostIdentifikator(registreringIdentifikatorRequestType);
        }
        return endreStatusRequestType;
    }

    private static FagsystemType getFagSystemMapping(OppdaterJournalpostStatus input) {
        FagsystemType fagsystemType = new FagsystemType();
        BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
        if (input.getKontekst() != null) {
            //ToDo: Check mapping TD
            fagsystemType.setFagsystemNavn(input.getKontekst().getKlientnavn());
            brukeridentifikatorType.setBrukerident(input.getKontekst().getSluttbruker());
        }
        fagsystemType.setFagsystemSluttbruker(brukeridentifikatorType);
        return fagsystemType;
    }


    /**
     * Response mapping (MimeWS to endpoint)
     */
    public static OppdaterJournalpostStatusResponse oppJournalpostStatusResToSuFa(KvitteringType mimeWS) {
        //No elements to return in response

        return new OppdaterJournalpostStatusResponse();
    }

}
