package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.felles.xml_schema._2012_01.Saksnummer;
import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Saksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappeResponse;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.*;
import no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.OpprettSaksmappeRequestType;

import java.math.BigInteger;

import static no.vegvesen.saksarkiv.felles.enkle._1.HandlingType.LEGG_TIL;

/**
 @author vijayjadhav
 **/
/**
 * Geo-Endpoint to MimeWS Request mapping for NySaksMappe operation
 */
public class SaksMappe {


    /**
     * Request mapping from the endpoint to MimeWS server
     * */
    public static OpprettSaksmappeRequestType endpointToMimeOppSM(final NySaksmappe input) {

        RequestValidator.validateNySaksmappeRequest(input);

        OpprettSaksmappeType opprettSaksmappeType = new OpprettSaksmappeType();
        SakMetadataType sakMetadataType = new SakMetadataType();

        sakMetadataType.setTittel(input.getMappe().getTittel());
        sakMetadataType.setOffentligTittel(input.getMappe().getOffentligTittel());

        if (input.getMappe().getMappetype() != null) {
            sakMetadataType.setMappeType(input.getMappe().getMappetype().getKodeverdi());
        }

        if (input.getMappe().getTilleggsinformasjon() != null && (input.getMappe().getTilleggsinformasjon().getListe().size() > 0)) {
            input.getMappe().getTilleggsinformasjon().getListe().forEach(item -> {
                        MetadataType metadataType = new MetadataType();
                        MappeAttributterType mappeAttributterType = new MappeAttributterType();
                        metadataType.setNavn(item.getTilgangsgruppeNavn());

                        if (item.getInformasjonstype() != null) {
                            metadataType.setVerdi(item.getInformasjonstype().getKodeverdi());
                            metadataType.setBeskrivelse(item.getInformasjonstype().getKodebeskrivelse());
                        }
                        mappeAttributterType.getAttributt().add(metadataType);
                        sakMetadataType.setMappetypespesifikkeAttributter(mappeAttributterType);
                    });
        }

        if (input.getMappe().getSkjerming() != null) {

            //ToDo check 22, 220 in the doc
            HjemmelRequestType hjemmelRequestType = new HjemmelRequestType();
            SkjermingRequestType skjermingRequestType = new SkjermingRequestType();
            SkjermingskodeRequestType skjermingskodeRequestType = new SkjermingskodeRequestType();

            if (input.getMappe().getSkjerming().getTilgangsrestriksjon() != null) {
//                 set the value hjemmelRequestType.setNavn to "Empty" when the code is "U";
                if (input.getMappe().getSkjerming().getTilgangsrestriksjon().getKodeverdi().equalsIgnoreCase("U")
                && input.getMappe().getSkjerming().getSkjermingshjemmel()==null){
                    hjemmelRequestType.setNavn("Empty");
                }
                else{
                skjermingskodeRequestType.setNavn(input.getMappe().getSkjerming().getTilgangsrestriksjon().getKodeverdi());
                }
                skjermingskodeRequestType.setKorrespondanseparterSkjermet(input.getMappe().getSkjerming().getTilgangsrestriksjon().isErGyldig());
                hjemmelRequestType.setNavn(input.getMappe().getSkjerming().getSkjermingshjemmel());
            }

            skjermingskodeRequestType.setHjemmel(hjemmelRequestType);
            skjermingRequestType.setKode(skjermingskodeRequestType);
            skjermingRequestType.setHandling(LEGG_TIL);
            sakMetadataType.setSkjerming(skjermingRequestType);
        }

        if (!input.getMappe().isSkjermetTittel()) {
            sakMetadataType.setOffentligTittel("");
        } else {
            sakMetadataType.setOffentligTittel(input.getMappe().getOffentligTittel());
        }

        TilgangType tilgangType = new TilgangType();
        TillatelseType tillatelseType = new TillatelseType();
        TilgangsGruppeTilgangType tilgangsGruppeTilgangType = new TilgangsGruppeTilgangType();
        if (input.getMappe().getTilgangsgruppeNavn() != null) {
            tilgangsGruppeTilgangType.setGruppenavn(input.getMappe().getTilgangsgruppeNavn());
        }
        tilgangType.setTillatelse(tillatelseType);
        opprettSaksmappeType.setTilgang(tilgangType);

        if (input.getMappe().getKlasse() != null && input.getMappe().getKlasse().getListe() != null &&
                input.getMappe().getKlasse().getListe().size() > 0) {
            input.getMappe().getKlasse().getListe().forEach(item -> {
                if (item != null) {
                    KlassifikasjonsverdiType klassifikasjonsverdiType = new KlassifikasjonsverdiType();
                    klassifikasjonsverdiType.setVerdi(item.getKlasseID());
                    klassifikasjonsverdiType.setIndeks(new BigInteger(item.getRekkefoelge()));
                    sakMetadataType.getKlassifikasjonsverdi().add(klassifikasjonsverdiType);
                }
            });
            tillatelseType.setTilgangsgruppe(tilgangsGruppeTilgangType);
        }

        AnsvarType ansvarType = new AnsvarType();
        AnsvarsstedType ansvarsstedType = new AnsvarsstedType();
        ansvarsstedType.setAnsvarskode(input.getMappe().getAdministrativEnhetInit());
        ansvarsstedType.setNavn(input.getMappe().getAdministrativEnhet());
        ansvarType.setAnsvarligEnhet(ansvarsstedType);
        BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
        brukeridentifikatorType.setNavn(input.getMappe().getSaksansvarlig());
        brukeridentifikatorType.setBrukerident(input.getMappe().getSaksansvarligInit());
        ansvarType.setAnsvarligPerson(brukeridentifikatorType);
        sakMetadataType.setAnsvar(ansvarType);
        opprettSaksmappeType.setMetadata(sakMetadataType);

        OpprettSaksmappeRequestType opprettSaksmappeRequestType = new OpprettSaksmappeRequestType();
        FagsystemType fagsystemType = getFagSystemMapping(input);
        opprettSaksmappeRequestType.setFagsystem(fagsystemType);


        opprettSaksmappeRequestType.setSaksmappe(opprettSaksmappeType);

        return opprettSaksmappeRequestType;
    }

    private static FagsystemType getFagSystemMapping(NySaksmappe input) {
        FagsystemType fagsystemType = new FagsystemType();
        BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
        if (input.getKontekst() != null) {
            fagsystemType.setFagsystemNavn(input.getKontekst().getKlientnavn());
            brukeridentifikatorType.setBrukerident(input.getKontekst().getSluttbruker());
        }

        fagsystemType.setFagsystemSluttbruker(brukeridentifikatorType);
        return fagsystemType;
    }

    /**
     * Response mapping (MimeWS to endpoint)
     */
    public static NySaksmappeResponse nySaksmappeResponseToSuFa(MappeKvitteringType mimeResponse) {


        NySaksmappeResponse nySaksmappeResponse = new NySaksmappeResponse();
        String saksNummRes = mimeResponse.getMappeId().getSaksnummer();

        if (saksNummRes != null) {
            Saksnummer saksnummer = new Saksnummer();
            Saksmappe saksmappe = new Saksmappe();
            saksnummer.setSaksaar(new BigInteger(saksNummRes.substring(0, 2)));
            saksnummer.setSakssekvensnummer(new BigInteger(saksNummRes.substring(3, saksNummRes.length())));

            saksmappe.setSaksnr(saksnummer);
            saksmappe.setSystemID(mimeResponse.getMappeId().getSaksnummer());
            nySaksmappeResponse.setReturn(saksmappe);

        }
        return nySaksmappeResponse;
    }
}
