package no.integration.svv.utils.validator;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.JournpostSystemID;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyDokument;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.OppdaterJournalpostStatus;
import no.integration.svv.exception.ValidationException;

/**
 @author vijayjadhav
 **/

/**
 * Validator class to check SuFa to endpoint server request mapping for all 4 operation for mandatory fields
 * */
public class RequestValidator {

    public static final String NY_JOURNALPOST_ENDPOINT = "NyJournalpost";
    public static final String NY_SAKSMAPPE_ENDPOINT = "NySaksmappe";
    public static final String NY_DOKUMENT_ENDPOINT = "NyDokument";
    public static final String NY_JPSTATUS_ENDPOINT = "OppdaterJournalpostStatus";


    /**
     * Method to validate NyJournalpost request to MimeWS for mandatory fields
     * */
    public static void validateJournalpostRequest(NyJournalpost request) throws ValidationException {
        if (request == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "Body", "Missing value");

        }

        if (request.getJournalpost() == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getJournalpost", "Missing value");

        }

        if (request.getJournalpost().getKorrespondansepart() == null || request.getJournalpost().getKorrespondansepart().getListe().size() == 0) {

            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getKorrespondansepart", "Missing value");
        }
        if (request.getJournalpost().getKorrespondansepart().getListe().get(0).getKorrespondanseparttype() == null) {

            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getKorrespondanseparttype", "Missing value");
        }

        if (request.getJournalpost().getJournalposttype() == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getJournalposttype", "Missing value");

        }

        if (request.getKontekst() == null) {
            throw new ValidationException(NY_JOURNALPOST_ENDPOINT, "getKontekst", "Missing value");
        }

    }
    /**
     * Method to validate NySaksmappe request to MimeWS for mandatory fields
     * */
    //ToDO: Check for the required fields to validate
    public static void validateNySaksmappeRequest(NySaksmappe request) throws ValidationException {
        if (request == null) {
            throw new ValidationException(NY_SAKSMAPPE_ENDPOINT, "Body", "Missing value");
        }
        if (request.getMappe() == null) {
            throw new ValidationException(NY_SAKSMAPPE_ENDPOINT, "getMappe", "Missing value");
        }


    }



    /**
     * Method to validate NyDokument request to MimeWS for mandatory fields
     * */
    public static void validateNyDokumentRequest(NyDokument request) throws ValidationException {

        if (request == null) {
            throw new ValidationException(NY_DOKUMENT_ENDPOINT, "Body", "Missing value");

        }
        if (request.getDokument() == null) {
            throw new ValidationException(NY_DOKUMENT_ENDPOINT, "getDokument", "Missing value");

        }

        if (request.getDokument().getFil() == null) {
            throw new ValidationException(NY_DOKUMENT_ENDPOINT, "getFil", "Missing value");

        }

    }
    /**
     * Method to validate NyJournalpostStatus request to MimeWS for mandatory fields
     * */
    public static void validateJPStatusRequest(OppdaterJournalpostStatus request) throws ValidationException {

        if (request == null) {
            throw new ValidationException(NY_JPSTATUS_ENDPOINT, "Body", "Missing value");

        }
        if (request.getJournalstatus() == null) {
            throw new ValidationException(NY_JPSTATUS_ENDPOINT, "getJournalstatus", "Missing value");

        }
        if (request.getJournalpostnokkel() == null) {
            throw new ValidationException(NY_JPSTATUS_ENDPOINT, "getJournalpostnokkel", "Missing value");

        }
        JournpostSystemID journpostSystemID = (JournpostSystemID) request.getJournalpostnokkel();
        if (journpostSystemID.getSystemID() == null) {

            throw new ValidationException(NY_JPSTATUS_ENDPOINT, "getSystemID", "Missing value");

        }


    }


}
