package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.dokument.xml_schema._2012_01.Dokument;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyDokument;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyDokumentResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.NyttVedleggKvitteringType;
import no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.LeggTilNyttDokumentJournalpostRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;

class DokumentMapTest {

    @Test
    void endpointToMimeOppDok() throws UnsupportedEncodingException {


        NyDokument geoInput = DokumentMapTestFixture.ndRequest();
        LeggTilNyttDokumentJournalpostRequestType mimeOutput = DokumentMap.endpointToMimeOppDok(geoInput);

        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getFagsystem());
        Assertions.assertNotNull(mimeOutput.getDokument());

        mimeOutput.getDokument().forEach(item -> {
            Assertions.assertNotNull(item);
            Assertions.assertNotNull(item.getMetadata());
//            Assertions.assertNotNull(item.getDokumentrolle());
            Assertions.assertNotNull(item.getFil());
            Assertions.assertNotNull(item.getMetadata());

        });
        Assertions.assertNotNull(mimeOutput.getRegistreringIdentifikator());

        Assertions.assertEquals(mimeOutput.getRegistreringIdentifikator().getDokumentId(), geoInput.getDokument().getSystemID());
        mimeOutput.getDokument().forEach(item -> {
            Assertions.assertEquals(item.getMetadata().getDokumenttype(),geoInput.getDokument().getDokumenttype().getKodeverdi());
            Assertions.assertEquals(item.getMetadata().getTittel(),geoInput.getDokument().getTittel());
            Assertions.assertEquals(item.getFil().getFilnavn(),geoInput.getDokument().getFil().getFilnavn());
            Assertions.assertEquals(item.getFil().getMimetype(),geoInput.getDokument().getFil().getMimeType());

        });


    }
        @Test
        @DisplayName("Throw validation exception when Body is null")
        void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

            ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                DokumentMap.endpointToMimeOppDok(null);
            });
            Assertions.assertEquals("Missing value", thrown.getMessage());
            Assertions.assertEquals(RequestValidator.NY_DOKUMENT_ENDPOINT, thrown.getEndpointName());
            Assertions.assertEquals("Body", thrown.getFieldName());

        }

        @Test
        @DisplayName("Throw validation exception when Dokument is null")
        void dokumentIsNull() throws DatatypeConfigurationException, ParseException {

            NyDokument geoInput = DokumentMapTestFixture.ndRequest();
            geoInput.setDokument(null);

            ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
                DokumentMap.endpointToMimeOppDok(geoInput);
            });
            Assertions.assertEquals("Missing value", thrown.getMessage());
            Assertions.assertEquals(RequestValidator.NY_DOKUMENT_ENDPOINT, thrown.getEndpointName());
            Assertions.assertEquals("getDokument", thrown.getFieldName());
        }

    @Test
    @DisplayName("Throw validation exception when Fil is null")
    void filIsNull() throws DatatypeConfigurationException, ParseException {

        NyDokument geoInput = DokumentMapTestFixture.ndRequest();
        Dokument dokument = new Dokument();
        dokument.setFil(null);
        geoInput.setDokument(dokument);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            DokumentMap.endpointToMimeOppDok(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_DOKUMENT_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getFil", thrown.getFieldName());
    }


    @Test
    void nyDokumentResponseToSuFa() {

        NyttVedleggKvitteringType mimeInput = DokumentMapTestFixture.ndResponse();

        NyDokumentResponse geoOutput = DokumentMap.nyDokumentResponseToSuFa(mimeInput);

        Assertions.assertNotNull(geoOutput);
        Assertions.assertNotNull(geoOutput.getReturn());

        mimeInput.getDokument().forEach(item->{
            Assertions.assertEquals(geoOutput.getReturn().getDokumentnummer(),item.getDokumentId());
        });

    }




}