package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.Journalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpost;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyJournalpostResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.FagsystemType;
import no.vegvesen.saksarkiv.felles.typer._1.JournalpostKvitteringType;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.OpprettJournalpostRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.text.ParseException;

class JournalPostTest {

    @Test
    void nyJournalPostRequestToMime() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();

        OpprettJournalpostRequestType mimeOutput = JournalPost.endpointToMimeOppJPT(geoInput);
        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getJournalpost());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getMappe());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getMappe().getMappeId());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter().getAttributt());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getRegistreringstypespesifikkeAtributter().getAttributt().get(0));
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming().getKode());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getBasisMetadata().getSkjerming().getKode().getHjemmel());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getBruker());

        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getGruppe());

        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getOrgenhet());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getTilgang().getTillatelse().getTilgangsgruppe());
        Assertions.assertNotNull(mimeOutput.getJournalpost().getVedlegg());

        Assertions.assertEquals(geoInput.getJournalpost().getTittel(), mimeOutput.getJournalpost().getBasisMetadata().getTittel());
        Assertions.assertEquals(geoInput.getJournalpost().getOffentligTittel(), mimeOutput.getJournalpost().getBasisMetadata().getOffentligTittel());
        Assertions.assertEquals(geoInput.getJournalpost().getTilgangsgruppeNavn(), mimeOutput.getJournalpost().getTilgang().getTillatelse().getTilgangsgruppe().getGruppenavn());
    }


    @Test
    @DisplayName("Throw validation exception when Body is null")
    void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when JournalPost is null")
    void nyJournalPostIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        geoInput.setJournalpost(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getJournalpost", thrown.getFieldName());
    }




    @Test
    @DisplayName("Throw validation exception when JournalPost.getKontekst is null")
    void KontekstIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        geoInput.setKontekst(null);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getKontekst", thrown.getFieldName());
    }


    @Test
    @DisplayName("Throw validation exception when JournalPost.getKorrespondansepart is null")
    void korrespondansepartlIsNull() throws DatatypeConfigurationException, ParseException {

        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        Journalpost journalpost = new Journalpost();

        journalpost.setKorrespondansepart(null);
        geoInput.setJournalpost(journalpost);
        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            JournalPost.endpointToMimeOppJPT(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JOURNALPOST_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getKorrespondansepart", thrown.getFieldName());
    }

    @Test
    void nyJournalpostResponseToSuFa() throws DatatypeConfigurationException, ParseException {

        /*Set up* */
        JournalpostKvitteringType mimeInput  = JournalPostTestFixture.jpResponse();
        NyJournalpostResponse  geoOutput = JournalPost.nyJournalpostResponseToSuFa(mimeInput);

        Assertions.assertNotNull(geoOutput);
        Assertions.assertNotNull(geoOutput.getReturn());
        Assertions.assertNotNull(geoOutput.getReturn().getJournalpostnummer());

//        Assert
        Assertions.assertEquals(geoOutput.getReturn().getSystemID(), mimeInput.getRegistrering().getDokumentId());
        Assertions.assertEquals(geoOutput.getReturn().getJournalpostnummer(), mimeInput.getRegistrering().getJournalpostId().getLopenummer());

        String saksNumRes = mimeInput.getRegistrering().getJournalpostId().getMappeId().getSaksnummer();

    }



    @Test
    void getFagSystemMapping() throws DatatypeConfigurationException, ParseException {
        NyJournalpost geoInput = JournalPostTestFixture.jpRequest();
        FagsystemType fagSystemMapping = JournalPost.getFagSystemMapping(geoInput);

        Assertions.assertNotNull(fagSystemMapping);
        Assertions.assertNotNull(fagSystemMapping.getFagsystemSluttbruker());



    }


}

