package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.JournpostSystemID;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.OppdaterJournalpostStatus;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.EndreStatusRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.text.ParseException;

class OppJournalpostStatusTest {

    @Test
    void endpointToMimeOppJPStatus() {

        OppdaterJournalpostStatus geoInput = OppJournalpostStatusTestFixure.oJpStatusRequest();

        EndreStatusRequestType mimeOutput = OppJournalpostStatus.endpointToMimeOppJPStatus(geoInput);

        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getJournalpostIdentifikator());

        JournpostSystemID journpostSystemID = (JournpostSystemID) geoInput.getJournalpostnokkel();
        Assertions.assertEquals(journpostSystemID.getSystemID().getId(),mimeOutput.getJournalpostIdentifikator().getDokumentId());

    }

    @Test
    @DisplayName("Throw validation exception when Body is null")
    void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            OppJournalpostStatus.endpointToMimeOppJPStatus(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JPSTATUS_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when getJournalpostnokkel is null")
    void journalpostnokkelIsNull() throws DatatypeConfigurationException, ParseException {

        OppdaterJournalpostStatus geoInput = OppJournalpostStatusTestFixure.oJpStatusRequest();

        JournpostSystemID journpostSystemID = (JournpostSystemID) geoInput.getJournalpostnokkel();
        geoInput.setJournalpostnokkel(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            OppJournalpostStatus.endpointToMimeOppJPStatus(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JPSTATUS_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getJournalpostnokkel", thrown.getFieldName());
    }

    @Test
    @DisplayName("Throw validation exception when getSystemID is null")
    void systemIDIsNull() throws DatatypeConfigurationException, ParseException {


        OppdaterJournalpostStatus geoInput = OppJournalpostStatusTestFixure.oJpStatusRequest();

        JournpostSystemID journpostSystemID = (JournpostSystemID) geoInput.getJournalpostnokkel();
        journpostSystemID.setSystemID(null);
        geoInput.setJournalpostnokkel(journpostSystemID);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            OppJournalpostStatus.endpointToMimeOppJPStatus(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_JPSTATUS_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getSystemID", thrown.getFieldName());
    }


    /*
    * Response: Has no elements in response
     */
}