package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappe;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappeResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;
import no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.OpprettSaksmappeRequestType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import javax.xml.datatype.DatatypeConfigurationException;
import java.math.BigInteger;
import java.text.ParseException;

class SaksMappeTest {

    @Test
    void endpointToMimeOppSM() throws DatatypeConfigurationException, ParseException {
        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();

        OpprettSaksmappeRequestType mimeOutput = SaksMappe.endpointToMimeOppSM(geoInput);

        Assertions.assertNotNull(mimeOutput);
        Assertions.assertNotNull(mimeOutput.getFagsystem());
        Assertions.assertNotNull(mimeOutput.getFagsystem().getFagsystemSluttbruker());

        Assertions.assertNotNull(mimeOutput.getSaksmappe());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang().getTillatelse());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getTilgang().getTillatelse().getTilgangsgruppe());

        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getHjemmel());
        Assertions.assertNotNull(mimeOutput.getSaksmappe().getMetadata().getMappetypespesifikkeAttributter().getAttributt());


        Assertions.assertEquals(geoInput.getMappe().getTittel(), mimeOutput.getSaksmappe().getMetadata().getTittel());
        Assertions.assertEquals(geoInput.getMappe().getMappetype().getKodeverdi(), mimeOutput.getSaksmappe().getMetadata().getMappeType());
        Assertions.assertEquals(geoInput.getMappe().getAdministrativEnhetInit(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligEnhet().getAnsvarskode());
        Assertions.assertEquals(geoInput.getMappe().getAdministrativEnhet(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligEnhet().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSaksansvarlig(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligPerson().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSaksansvarligInit(), mimeOutput.getSaksmappe().getMetadata().getAnsvar().getAnsvarligPerson().getBrukerident());


        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getTilgangsrestriksjon().getKodeverdi(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getTilgangsrestriksjon().isErGyldig(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().isKorrespondanseparterSkjermet());
        Assertions.assertEquals(geoInput.getMappe().getSkjerming().getSkjermingshjemmel(), mimeOutput.getSaksmappe().getMetadata().getSkjerming().getKode().getHjemmel().getNavn());
        Assertions.assertEquals(geoInput.getMappe().getTilgangsgruppeNavn(), mimeOutput.getSaksmappe().getTilgang().getTillatelse().getTilgangsgruppe().getGruppenavn());


    }

    @Test
    @DisplayName("Throw validation exception when Body is null")
    void nyGeoInputIsNull() throws DatatypeConfigurationException, ParseException {

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.endpointToMimeOppSM(null);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("Body", thrown.getFieldName());

    }

    @Test
    @DisplayName("Throw validation exception when SaksMappe is null")
    void nySaksmappeIsNull() throws DatatypeConfigurationException, ParseException {

        NySaksmappe geoInput = SaksmappeTestFixture.saksmappeRequest();
        geoInput.setMappe(null);

        ValidationException thrown = Assertions.assertThrows(ValidationException.class, () -> {
            SaksMappe.endpointToMimeOppSM(geoInput);
        });
        Assertions.assertEquals("Missing value", thrown.getMessage());
        Assertions.assertEquals(RequestValidator.NY_SAKSMAPPE_ENDPOINT, thrown.getEndpointName());
        Assertions.assertEquals("getMappe", thrown.getFieldName());
    }



    @Test
    void nySaksmappeResponseToSuFa() throws DatatypeConfigurationException, ParseException {

        MappeKvitteringType mimeInput = SaksmappeTestFixture.saksmappeResponse();
        NySaksmappeResponse geoOutput = SaksMappe.nySaksmappeResponseToSuFa(mimeInput);

        Assertions.assertNotNull(mimeInput);

        String saksNumRes = mimeInput.getMappeId().getSaksnummer();
        Assertions.assertEquals(geoOutput.getReturn().getSaksnr().getSaksaar(), (new BigInteger(saksNumRes.substring(0, 2))));
        Assertions.assertEquals(geoOutput.getReturn().getSaksnr().getSakssekvensnummer(), (new BigInteger(saksNumRes.substring(3, saksNumRes.length()))));

    }




    @Test
    public void nySaksmappeResponseTest() throws DatatypeConfigurationException, ParseException {

        MappeKvitteringType mimeInput = SaksmappeTestFixture.saksmappeResponse();
        NySaksmappeResponse geoOutput = SaksMappe.nySaksmappeResponseToSuFa(mimeInput);

        Assertions.assertNotNull(geoOutput);
        Assertions.assertNotNull(geoOutput.getReturn());
        Assertions.assertNotNull(geoOutput.getReturn().getSaksnr());

        String saksNumRes = mimeInput.getMappeId().getSaksnummer();
        Assertions.assertEquals(new BigInteger(saksNumRes.substring(0, 2)), geoOutput.getReturn().getSaksnr().getSaksaar());
        Assertions.assertEquals(new BigInteger(saksNumRes.substring(3,saksNumRes.length())), geoOutput.getReturn().getSaksnr().getSakssekvensnummer());

    }
}