package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.felles.xml_schema._2012_01.Saksnummer;
import no.geointegrasjon.rep.arkiv.kjerne.xml_schema._2012_01.*;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NySaksmappe;
import no.vegvesen.saksarkiv.felles.typer._1.MappeIdType;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

public class SaksmappeTestFixture {


    public static NySaksmappe saksmappeRequest() throws DatatypeConfigurationException, ParseException {
        NySaksmappe geoInput = new NySaksmappe();
        Saksmappe saksmappe = new Saksmappe();

        Date date = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").parse("2022-01-14 11:15:00");

        saksmappe.setTittel("TittleCase");
        saksmappe.setSaksdato(dateConv(date));
        saksmappe.setOffentligTittel("Offtittle");
        saksmappe.setSkjermetTittel(false);
        saksmappe.setAdministrativEnhetInit("FGA40");
        saksmappe.setAdministrativEnhet("FGA40");
        saksmappe.setBevaringstid("2");
        saksmappe.setKassasjonsdato(dateConv(date));
        saksmappe.setSaksansvarlig("tytors");
        saksmappe.setSaksansvarligInit("tytors");
        saksmappe.setTilgangsgruppeNavn("Alle ansatte");
        Saksnummer saksnummer = new Saksnummer();
        saksnummer.setSakssekvensnummer(new BigInteger("9876787"));
        saksnummer.setSaksaar(new BigInteger("22"));


        Mappetype mappetype = new Mappetype();
        mappetype.setErGyldig(true);
        mappetype.setKodeverdi("Sak");

        KlasseListe klasseListe = new KlasseListe();

        Klasse klasse = new Klasse();
        klasse.setKlasseID("100");
        klasse.setTittel("KlassTittle");
        klasse.setRekkefoelge("1");
        Klassifikasjonssystem klassifikasjonssystem = new Klassifikasjonssystem();
        klassifikasjonssystem.setKodeverdi("Ignored");
        klassifikasjonssystem.setErGyldig(false);
        klasse.setKlassifikasjonssystem(klassifikasjonssystem);
        klasseListe.getListe().add(klasse);

        Skjerming skjerming = new Skjerming();
        skjerming.setSkjermingshjemmel("§ 24, 3. ledd, 2. punktum");

        Tilgangsrestriksjon tilgangsrestriksjon = new Tilgangsrestriksjon();
        tilgangsrestriksjon.setKodeverdi("Alle");
        tilgangsrestriksjon.setErGyldig(false);
        skjerming.setTilgangsrestriksjon(tilgangsrestriksjon);
        skjerming.setSkjermingOpphoererDato(dateConv(date));


        saksmappe.setKlasse(klasseListe);
        saksmappe.setMappetype(mappetype);
        saksmappe.setSaksnr(saksnummer);
        saksmappe.setSkjerming(skjerming);
        EksternNoekkel eksternNoekkel = new EksternNoekkel();
        eksternNoekkel.setNoekkel("tytors");
        eksternNoekkel.setFagsystem("Vadis");
        saksmappe.setReferanseEksternNoekkel(eksternNoekkel);
        Tilleggsinformasjon tilleggsinformasjon = new Tilleggsinformasjon();
        tilleggsinformasjon.setInformasjon("TillInfo");
        tilleggsinformasjon.setOppbevaresTilDato(dateConv(date));
        tilleggsinformasjon.setRegistrertAvInit("RehAvInit");
        tilleggsinformasjon.setTilgangsgruppeNavn("Alle");
        tilleggsinformasjon.setRegistrertDato(dateConv(date));


        TilleggsinformasjonListe tilleggsinformasjonListe = new TilleggsinformasjonListe();
        tilleggsinformasjonListe.getListe().add(tilleggsinformasjon);
        saksmappe.setTilleggsinformasjon(tilleggsinformasjonListe);


        geoInput.setMappe(saksmappe);
        return geoInput;


    }

    public static MappeKvitteringType saksmappeResponse() throws DatatypeConfigurationException, ParseException {

        MappeKvitteringType mimeInput = new MappeKvitteringType();

        MappeIdType mappeIdType = new MappeIdType();
        mappeIdType.setSaksnummer("22/323234");
        mimeInput.setMappeId(mappeIdType);

        return mimeInput;


    }

    public static XMLGregorianCalendar dateConv(Date dt) throws DatatypeConfigurationException, ParseException {
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        Date date = format.parse("2022-01-14 11:15:00");
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(date);
        XMLGregorianCalendar xmlGregCal = DatatypeFactory.newInstance().newXMLGregorianCalendar(cal);
//        System.out.println(xmlGregCal);
        return xmlGregCal;

    }
}

